#include "goodmanwearehistogrambuilder.h"
#include "metropolishistogrambuilder.h"
#include "function.h"
#include "randomnumbergenerator.h"
#include <stdlib.h>
#include <iostream>
#include <cmath>

using namespace meanwalker;

// WARNING: This is an insanely BAD random number generator. For real
//          applications, use a decent one (e.g. the Mersenne twister,
//          available in the GNU Scientific Library)

class MyRandomNumberGenerator : public RandomNumberGenerator
{
public:
	MyRandomNumberGenerator()	{ }
	~MyRandomNumberGenerator()	{ }

	double pickRandomNumber()
	{
		double randomNumber = (1.0*((double)rand()/(RAND_MAX + 1.0)));

		return randomNumber;
	}
};

// This is the logarithm of the probability density we're going to
// sample: a gaussian distribution with specific mean and standard
// deviation

class LogGaussianFunction : public Function
{
public:
	LogGaussianFunction(double center, double sigma)
	{
		m_center = center;
		m_sigma = sigma;
	}

	size_t getDimension() const
	{
		return 1;
	}

	double operator()(const double *pCoords) const
	{
		double x = pCoords[0];
		double diff = (x-m_center)/m_sigma;
		return -diff*diff/2.0;
	}

	bool isLog() const
	{
		return true;
	}
private:
	double m_center, m_sigma;
};

int main(void)
{
	MyRandomNumberGenerator rndGen;
	GoodmanWeareHistogramBuilder builder1(&rndGen);
	MetropolisHistogramBuilder builder2(&rndGen);
	LogGaussianFunction f(10, 3);

	double startCoord[] = { 2 };
	double minc[] = { -20 };
	double maxc[] = { 30 };
	size_t numBins = 100;

	// First we'll sample the distribution using the Goodman-Weare method.
	// This will create a histogram with boundaries specfied by minc and maxc

	if (!builder1.init(&f, startCoord, 1.0, numBins, minc, maxc))
	{
		std::cerr << builder1.getErrorString() << std::endl;
		return -1;
	}

	builder1.run(10000, 1000000);
	builder1.printHistograms();

	// Then we'll do the same with the Metropolis-Hastings method.

	if (!builder2.init(&f, minc, maxc, startCoord, numBins))
	{
		std::cerr << builder2.getErrorString() << std::endl;
		return -1;
	}

	builder2.run(10000, 1000000);
	builder2.printHistograms();

	return 0;
}

