/*

  This file is a part of MeanWalker, a library which provides utilities to
  sample from probability distributions using methods like the 
  Metropilis-Hastings algorithm and Goodman-Weare algorithm.

  Copyright (C) 2012 Jori Liesenborgs

  Contact: jori.liesenborgs@gmail.com
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
*/

#include "metropolishistogrambuilder.h"
#include <iostream>

namespace meanwalker
{

MetropolisHistogramBuilder::MetropolisHistogramBuilder(RandomNumberGenerator *pRndGen) : HistogramBuilderTemplate<MetropolisWalker>(pRndGen)
{
}

MetropolisHistogramBuilder::~MetropolisHistogramBuilder()
{
}

bool MetropolisHistogramBuilder::init(Function *pProbFunction, const double *pMinCoords, const double *pMaxCoords, 
		                      const double *pStartCoords, size_t numBins, bool keepInside,
				      bool delayed)
{
	bool retValue;

	if (delayed)
		retValue = initDelayed(pProbFunction->getDimension(), numBins);
	else
		retValue = initNormal(pMinCoords, pMaxCoords, pProbFunction->getDimension(), numBins);

	if (!retValue)
		return false;

	if (!MetropolisWalker::init(pProbFunction, pMinCoords, pMaxCoords, pStartCoords, keepInside))
	{
		destroy();
		return false;
	}

	return true;
}

} // end namespace   

